package com.diagbot.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.BehospitalInfoDTO;
import com.diagbot.dto.DeptNumDTO;
import com.diagbot.dto.LevelStatisticsDTO;
import com.diagbot.dto.MsgDTO;
import com.diagbot.dto.NumDTO;
import com.diagbot.entity.BehospitalInfo;
import com.diagbot.vo.AnalyzeCodeVO;
import com.diagbot.vo.AnalyzeVO;
import com.diagbot.vo.BehospitalPageVO;
import com.diagbot.vo.FilterOrderByDeptVO;
import com.diagbot.vo.FilterOrderVO;
import com.diagbot.vo.FilterPageByDeptVO;
import com.diagbot.vo.FilterPageVO;
import com.diagbot.vo.FilterVO;
import com.diagbot.vo.HomePageFilterVO;
import org.apache.ibatis.annotations.Param;
import com.diagbot.vo.TaskVO;

import java.util.List;

/**
 * <p>
 * 住院病历信息 Mapper 接口
 * </p>
 *
 * @author zhoutg
 * @since 2020-04-13
 */
public interface BehospitalInfoMapper extends BaseMapper<BehospitalInfo> {

    public IPage<BehospitalInfoDTO> getPage(BehospitalPageVO behospitalPageVO);

    public List<MsgDTO> getMsg(AnalyzeVO analyzeVO);

    public List<MsgDTO> getMsgByEntryCode(AnalyzeCodeVO analyzeCodeVO);

    /**
     * 缺陷排行列表统计
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> resultStatistics(FilterVO filterVO);

    /**
     * 各科室缺陷占比
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> resultStatisticsByDept(FilterVO filterVO);

    /**
     * 缺陷排行列表统计-百分比未计算
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> resultStatistics2(FilterVO filterVO);

    /**
     * 各科室缺陷占比-百分比未计算
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> resultStatisticsByDept2(FilterVO filterVO);

    /**
     * 缺陷总数
     *
     * @param filterVO
     * @return
     */
    public int getTotleResultNum(FilterVO filterVO);

    public List<BehospitalInfo> getNoGrade(TaskVO taskVO);

    IPage<BehospitalInfoDTO> getPageByDept(BehospitalPageVO behospitalPageVO);

    IPage<BehospitalInfoDTO> getPageByPerson(BehospitalPageVO behospitalPageVO);

    /**
     * 病案首页相关病历数统计
     *
     * @param homePageFilterVO
     * @return
     */
    public int homePageCount(HomePageFilterVO homePageFilterVO);

    /**
     * 手术病历数统计
     *
     * @param filterVO
     * @return
     */
    public int homePageCountForOperation(FilterVO filterVO);

    /**
     * 按科室统计病案首页病历数
     *
     * @param homePageFilterVO
     * @return
     */
    public List<NumDTO> homePageCountByDept(HomePageFilterVO homePageFilterVO);

    /**
     * 按科室统计病案首页手术病历数
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> homePageCountForOperationByDept(FilterVO filterVO);

    /**
     * 各科室缺陷占比排行（分页）
     *
     * @param filterPageVO
     * @return
     */
    public IPage<NumDTO> resultStatisticsByDeptPage(@Param("filterPageVO") FilterPageVO filterPageVO);

    /**
     * 各科室缺陷占比（组合）
     *
     * @param filterOrderVO
     * @return
     */
    public List<LevelStatisticsDTO> levelStatistics(FilterOrderVO filterOrderVO);

    /**
     * 病案首页合格率占比
     *
     * @param filterOrderVO
     * @return
     */
    public List<LevelStatisticsDTO> homePageLevelStatistics(FilterOrderVO filterOrderVO);

    /**
     * 各科室缺陷占比（主任医生）
     *
     * @param filterOrderByDeptVO
     * @return
     */
    public List<LevelStatisticsDTO> levelStatisticsByDept(FilterOrderByDeptVO filterOrderByDeptVO);

    /**
     * 病案首页合格率占比（主任医生）
     *
     * @param filterOrderByDeptVO
     * @return
     */
    public List<LevelStatisticsDTO> homePageLevelStatisticsByDept(FilterOrderByDeptVO filterOrderByDeptVO);

    /**
     * 科室缺陷占比-科室（分页）
     *
     * @param filterPageByDeptVO
     * @return
     */
    public IPage<DeptNumDTO> resultStatisticsByDeptAndDoctorPage(@Param("filterPageByDeptVO") FilterPageByDeptVO filterPageByDeptVO);

    /**
     * 各科室缺陷占比（组合）-全院-根据内外科系统统计
     *
     * @param filterOrderVO
     * @return
     */
    public List<LevelStatisticsDTO> levelStatisticsByDeptClass(FilterOrderVO filterOrderVO);
}